@@ -96,8 +96,8 @@ class IndexParser:
 
     @property
     def releaselinks(self):
-        l = map(BasenameMeta, self.basename2link.values())
-        return [x.obj for x in l]
+        # the BasenameMeta wrapping essentially does link validation
+        return [BasenameMeta(x).obj for x in self.basename2link.values()]
 
     def parse_index(self, disturl, html):
         p = HTMLPage(html, disturl.url)
@@ -360,12 +360,15 @@ class PyPIStage(BaseStage):
             # maintain list of currently cached project names to enable
             # deletion and offline mode
             self.add_project_name(project)
-        # XXX if the transaction fails the links are still marked
-        # as refreshed but the data was not persisted.  It's a rare
-        # enough event (tm) to not worry too much, though.
-        # (we can, however, easily add a
-        # keyfs.tx.on_commit_success(callback) method.
-        self.cache_retrieve_times.refresh(project)
+
+        def on_commit():
+            threadlog.info("setting projects cache for %r", project)
+            self.cache_retrieve_times.refresh(project)
+            # make project appear in projects list even
+            # before we next check up the full list with remote
+            self.cache_projectnames.add(project)
+
+        self.keyfs.tx.on_commit_success(on_commit)
 
     def _load_cache_links(self, project):
         is_expired, links_with_data, serial = True, None, -1
@@ -477,7 +480,7 @@ class PyPIStage(BaseStage):
         # parse simple index's link
         assert response.text is not None, response.text
         result = parse_index(response.url, response.text)
-        releaselinks = list(result.releaselinks)
+        releaselinks = result.releaselinks
 
         # first we try to process mirror links without an explicit write transaction.
         # if all links already exist in storage we might then return our already
@@ -495,10 +498,6 @@ class PyPIStage(BaseStage):
             requires_python = [link.requires_python for link in releaselinks]
             yanked = [link.yanked for link in releaselinks]
             self._save_cache_links(project, links, requires_python, yanked, serial)
-            # make project appear in projects list even
-            # before we next check up the full list with remote
-            threadlog.info("setting projects cache for %r", project)
-            self.cache_projectnames.get_inplace().add(project)
             return join_links_data(links, requires_python, yanked)
 
         try:
@@ -601,7 +600,7 @@ class ProjectNamesCache:
     """ Helper class for maintaining project names from a mirror. """
     def __init__(self):
         self._timestamp = -1
-        self._data = set()
+        self._data = frozenset()
 
     def exists(self):
         return self._timestamp != -1
@@ -611,16 +610,20 @@ class ProjectNamesCache:
 
     def get(self):
         """ Get a copy of the cached data. """
-        return set(self._data)
-
-    def get_inplace(self):
-        """ Get cached data in-place. """
         return self._data
 
+    def add(self, project):
+        """ Add project to cache. """
+        self._data = self._data.union({project})
+
+    def discard(self, project):
+        """ Remove project from cache. """
+        self._data = self._data.difference({project})
+
     def set(self, data):
         """ Set data and update timestamp. """
         if data is not self._data:
-            self._data = data.copy()
+            self._data = frozenset(data)
         self.mark_current()
 
     def mark_current(self):
