@@ -572,8 +572,11 @@ class PyPIView:
             whitelist_info = stage.get_mirror_whitelist_info(project)
             embed_form = whitelist_info['has_mirror_base']
             blocked_index = whitelist_info['blocked_by_mirror_whitelist']
-        response = Response(body=b"".join(self._simple_list_project(
-            stage, project, result, embed_form, blocked_index)))
+        body = b"".join(self._simple_list_project(
+            stage, project, result, embed_form, blocked_index))
+        response = Response(
+            content_length=len(body),
+            body=body)
         if stage.ixconfig['type'] == 'mirror':
             serial = stage.key_projsimplelinks(project).get().get("serial")
             if serial > 0:
@@ -581,9 +584,6 @@ class PyPIView:
         return response
 
     def _simple_list_project(self, stage, project, result, embed_form, blocked_index):
-        response = self.request.response
-        response.content_type = "text/html ; charset=utf-8"
-
         title = "%s: links for %s" % (stage.name, project)
         yield ("<!DOCTYPE html><html><head><title>%s</title></head><body><h1>%s</h1>\n" %
                (title, title)).encode("utf-8")
@@ -659,17 +659,20 @@ class PyPIView:
         self.log.info("starting +simple")
         stage = self.context.stage
         try:
+            # list is called to force iteration over all results in this
+            # try/except block
             stage_results = list(stage.list_projects())
         except stage.UpstreamError as e:
             threadlog.error(e.msg)
             abort(self.request, 502, e.msg)
         # at this point we are sure we can produce the data without
         # depending on remote networks
-        return Response(body=b"".join(self._simple_list_all(stage, stage_results)))
+        body = b"".join(self._simple_list_all(stage, stage_results))
+        return Response(
+            content_length=len(body),
+            body=body)
 
     def _simple_list_all(self, stage, stage_results):
-        response = self.request.response
-        response.content_type = "text/html ; charset=utf-8"
         title = "%s: simple list (including inherited indices)" % (stage.name)
         yield ("<!DOCTYPE html><html><head><title>%s</title></head><body><h1>%s</h1>" %(
               title, title)).encode("utf-8")
@@ -702,6 +705,8 @@ class PyPIView:
 
     @view_config(
         route_name="/{user}/{index}", request_method="PUT")
+    @view_config(
+        route_name="/{user}/{index}/", request_method="PUT")
     def index_create(self):
         username = self.context.username
         user = self.model.get_user(username)
@@ -748,6 +753,9 @@ class PyPIView:
     @view_config(
         route_name="/{user}/{index}", request_method="PATCH",
         permission="index_modify")
+    @view_config(
+        route_name="/{user}/{index}/", request_method="PATCH",
+        permission="index_modify")
     def index_modify(self):
         stage = self.context.stage
         json = getjson(self.request)
@@ -775,6 +783,8 @@ class PyPIView:
                 else:
                     raise ValueError("Unknown operator '%s'." % op)
             json = ixconfig
+        if json.get('type') == 'indexconfig' and 'result' in json:
+            json = json['result']
         oldconfig = dict(stage.ixconfig)
         try:
             ixconfig = stage.modify(**json)
@@ -798,6 +808,9 @@ class PyPIView:
     @view_config(
         route_name="/{user}/{index}", request_method="DELETE",
         permission="index_delete")
+    @view_config(
+        route_name="/{user}/{index}/", request_method="DELETE",
+        permission="index_delete")
     def index_delete(self):
         stage = self.context.stage
         if not stage.ixconfig["volatile"]:
@@ -807,6 +820,7 @@ class PyPIView:
         apireturn(201, "index %s deleted" % stage.name)
 
     @view_config(route_name="/{user}/{index}", request_method=("POST", "PUSH"))
+    @view_config(route_name="/{user}/{index}/", request_method=("POST", "PUSH"))
     def pushrelease(self):
         request = self.request
         if request.POST.get(':action'):
@@ -849,7 +863,7 @@ class PyPIView:
             auth_user = request.authenticated_userid
             self.log.debug("targetindex %r, auth_user %r", targetindex,
                            auth_user)
-            if not request.has_permission("pypi_submit", context=target_stage):
+            if not request.has_permission("upload", context=target_stage):
                 apireturn(401, message="user %r cannot upload to %r"
                                       %(auth_user, targetindex))
             self._set_versiondata_dict(target_stage, metadata)
@@ -982,7 +996,7 @@ class PyPIView:
         stage = self.context.stage
         if not hasattr(stage, "store_releasefile"):
             abort_submit(request, 404, "cannot submit to mirror index")
-        if not request.has_permission("pypi_submit"):
+        if not request.has_permission("upload"):
             # if there is no authenticated user, then issue a basic auth challenge
             if not request.authenticated_userid:
                 response = HTTPUnauthorized()
@@ -1298,6 +1312,7 @@ class PyPIView:
         apireturn(200, "package %r deleted" % relpath)
 
     @view_config(route_name="/{user}/{index}", accept="application/json", request_method="GET")
+    @view_config(route_name="/{user}/{index}/", accept="application/json", request_method="GET")
     def index_get(self):
         stage = self.context.stage
         result = dict(stage.ixconfig)
@@ -1339,6 +1354,9 @@ class PyPIView:
     @view_config(
         route_name="/{user}", request_method="PATCH",
         permission="user_modify")
+    @view_config(
+        route_name="/{user}/", request_method="PATCH",
+        permission="user_modify")
     def user_patch(self):
         request = self.request
         kvdict = getjson(request)
@@ -1358,6 +1376,9 @@ class PyPIView:
     @view_config(
         route_name="/{user}", request_method="PUT",
         permission="user_create")
+    @view_config(
+        route_name="/{user}/", request_method="PUT",
+        permission="user_create")
     def user_create(self):
         username = self.context.username
         request = self.request
@@ -1378,6 +1399,9 @@ class PyPIView:
     @view_config(
         route_name="/{user}", request_method="DELETE",
         permission="user_delete")
+    @view_config(
+        route_name="/{user}/", request_method="DELETE",
+        permission="user_delete")
     def user_delete(self):
         context = self.context
         if not context.user:
@@ -1393,6 +1417,7 @@ class PyPIView:
         apireturn(200, "user %r deleted" % context.username)
 
     @view_config(route_name="/{user}", accept="application/json", request_method="GET")
+    @view_config(route_name="/{user}/", accept="application/json", request_method="GET")
     def user_get(self):
         if self.context.user is None:
             apireturn(404, "user %r does not exist" % self.context.username)
@@ -1575,7 +1600,8 @@ def iter_fetch_remote_file(xom, entry):
     filestore = xom.filestore
     keyfs = xom.keyfs
     if not xom.is_replica():
-        keyfs.restart_as_write_transaction()
+        if not keyfs.tx.write:
+            keyfs.restart_as_write_transaction()
         entry = filestore.get_file_entry(entry.relpath, readonly=False)
         for part in iter_cache_remote_file(xom, entry):
             yield part
