@@ -155,10 +155,8 @@ def xom(request, makexom):
     return xom
 
 
-@pytest.fixture(autouse=True, scope="session")
-def speed_up_sqlite():
-    from devpi_server.keyfs_sqlite import Storage
-    old = Storage.ensure_tables_exist
+def _speed_up_sqlite(cls):
+    old = cls.ensure_tables_exist
 
     def make_unsynchronous(self, old=old):
         conn = old(self)
@@ -166,7 +164,14 @@ def speed_up_sqlite():
             conn._sqlconn.execute("PRAGMA synchronous=OFF")
         return
 
-    Storage.ensure_tables_exist = make_unsynchronous
+    cls.ensure_tables_exist = make_unsynchronous
+    return old
+
+
+@pytest.fixture(autouse=True, scope="session")
+def speed_up_sqlite():
+    from devpi_server.keyfs_sqlite import Storage
+    old = _speed_up_sqlite(Storage)
     yield
     Storage.ensure_tables_exist = old
 
@@ -174,15 +179,7 @@ def speed_up_sqlite():
 @pytest.fixture(autouse=True, scope="session")
 def speed_up_sqlite_fs():
     from devpi_server.keyfs_sqlite_fs import Storage
-    old = Storage.ensure_tables_exist
-
-    def make_unsynchronous(self, old=old):
-        conn = old(self)
-        with self.get_connection() as conn:
-            conn._sqlconn.execute("PRAGMA synchronous=OFF")
-        return
-
-    Storage.ensure_tables_exist = make_unsynchronous
+    old = _speed_up_sqlite(Storage)
     yield
     Storage.ensure_tables_exist = old
 
@@ -203,6 +200,8 @@ def storage_info(request):
     if backend is None:
         backend = 'devpi_server.keyfs_sqlite_fs'
     plugin = locate(backend)
+    if plugin is None:
+        raise RuntimeError("Couldn't find storage backend '%s'" % backend)
     result = plugin.devpiserver_storage_backend(settings=None)
     result["_test_plugin"] = plugin
     return result
@@ -222,6 +221,7 @@ def makexom(request, gentmp, httpget, monkeypatch, storage_info):
         from devpi_server import replica
         from devpi_server import view_auth
         from devpi_server import views
+        from devpi_server.interfaces import verify_connection_interface
         plugins = [
             plugin[0] if isinstance(plugin, tuple) else plugin
             for plugin in plugins]
@@ -263,6 +263,9 @@ def makexom(request, gentmp, httpget, monkeypatch, storage_info):
                 monkeypatch.setattr(extpypi.PyPIStage, "_get_remote_projects",
                     lambda self: set())
             add_pypistage_mocks(monkeypatch, httpget)
+        # verify storage interface
+        with xom.keyfs.get_connection() as conn:
+            verify_connection_interface(conn)
         # initialize default indexes
         from devpi_server.main import init_default_indexes
         if not xom.config.args.master_url:
@@ -471,9 +474,11 @@ class Mapp(MappMixin):
 
     def login(self, user="root", password="", code=200):
         api = self.getapi()
-        r = self.testapp.post_json(api.login,
-                                  {"user": user, "password": password},
-                                  expect_errors=True)
+        r = self.testapp.post_json(
+            api.login,
+            {"user": user, "password": password},
+            expect_errors=True,
+            headers={'Accept': 'application/json'})
         assert r.status_code == code
         if code == 200:
             password = r.json.get("result", r.json)["password"]
