@@ -118,6 +118,17 @@ def test_user_patch_keeps_missing_keys(testapp):
         'username': 'foo', 'title': 'foo', 'indexes': {}}
 
 
+def test_user_patch_trailing_slash(testapp):
+    # needed for devpi-client < 2.5.0
+    testapp.put_json("/foo", dict(password="123"))
+    testapp.set_auth('foo', '123')
+    r = testapp.get("/foo")
+    assert 'description' not in r.json['result']
+    testapp.patch_json("/foo/", dict(description="bar"))
+    r = testapp.get("/foo")
+    assert r.json['result']['description'] == 'bar'
+
+
 @pytest.mark.parametrize("nodeinfo,expected", [
     ({}, (None, None)),
     ({"uuid": "123", "role":"master"}, ("123", "123")),
@@ -197,6 +208,18 @@ def test_simple_project_redirect(pypistage, testapp):
     assert r.headers["location"].endswith("/root/pypi/+simple/%s/" % project)
 
 
+def test_index_get_json_patch_json_roundtrip(mapp, testapp):
+    api = mapp.create_and_use(indexconfig=dict(bases=["root/pypi"]))
+    r = testapp.get(api.index)
+    assert r.status_code == 200
+    assert r.json['type'] == 'indexconfig'
+    result = r.json['result']
+    result.pop('projects', None)
+    r = testapp.patch_json(api.index, r.json)
+    assert r.status_code == 200
+    assert r.json['result'] == result
+
+
 @pytest.mark.parametrize("outside_url", ['', 'http://localhost/devpi'])
 def test_simple_project_outside_url_subpath(mapp, outside_url, pypistage, testapp):
     api = mapp.create_and_use(indexconfig=dict(bases=["root/pypi"]))
